//! # Posts controller
//!
//! @author Pierre Hubert

use crate::api_data::post_api::PostAPI;
use crate::controllers::routes::RequestResult;
use crate::data::group::GroupAccessLevel;
use crate::data::http_request_handler::HttpRequestHandler;
use crate::data::post::{PostAccessLevel, PostPageKind};
use crate::helpers::{groups_helper, posts_helper, user_helper};

/// Get the list of posts of a user
pub fn get_list_user(r: &mut HttpRequestHandler) -> RequestResult {
    let user_id = r.post_user_id("userID")?;
    let start_from = r.post_u64_opt("startFrom", 0)?;

    if !user_helper::can_see_user_page(r.user_id_ref()?, &user_id)? {
        r.forbidden("You are not allowed to access this user posts !".to_string())?;
    }

    let posts = posts_helper::PostsQuery::new(r.user_id_opt())
        .set_start_from(start_from)
        .get_user(&user_id)?;

    r.set_response(PostAPI::for_list(&posts, r.user_id_opt())?)
}

/// Get the list of posts of a group
pub fn get_list_group(r: &mut HttpRequestHandler) -> RequestResult {
    let group_id = r.post_group_id_with_access("groupID", GroupAccessLevel::VIEW_ACCESS)?;
    let start_from = r.post_u64_opt("startFrom", 0)?;

    let posts = posts_helper::PostsQuery::new(r.user_id_opt())
        .set_start_from(start_from)
        .get_group(&group_id)?;

    r.set_response(PostAPI::for_list(&posts, r.user_id_opt())?)
}

/// Get the latest posts of a group
pub fn get_latest(r: &mut HttpRequestHandler) -> RequestResult {
    let start_from = r.post_u64_opt("startFrom", 0)?;
    let include_groups = r.post_bool_opt("include_groups", false);

    let posts = posts_helper::PostsQuery::new(r.user_id_opt())
        .set_start_from(start_from)
        .get_latest(include_groups)?;

    r.set_response(PostAPI::for_list(&posts, r.user_id_opt())?)
}

/// Get information about a single post
pub fn get_single(r: &mut HttpRequestHandler) -> RequestResult {
    let post = r.post_post_with_access("postID", PostAccessLevel::BASIC_ACCESS)?;

    r.set_response(PostAPI::new(&post, &r.user_id_opt())?)
}

/// Create a new post
pub fn create_post(r: &mut HttpRequestHandler) -> RequestResult {
    // Process page target
    let target_page = match r.post_string("kind-page")?.as_str() {
        "user" => {
            let user_id = r.post_user_id("kind-id")?;

            if !user_helper::can_create_posts(r.user_id_ref()?, &user_id)? {
                r.forbidden("You are not allowed to create posts on this page!".to_string())?;
            }

            PostPageKind::PAGE_KIND_USER(user_id)
        }

        "group" => {
            let group_id = r.post_group_id_with_access("kind-id", GroupAccessLevel::MEMBER_ACCESS)?;

            if !groups_helper::can_user_create_posts(&group_id, r.user_id_ref()?)? {
                r.forbidden("You are not allowed to create posts on this group!".to_string())?;
            }

            PostPageKind::PAGE_KIND_GROUP(group_id)
        }

        _ => {
            r.not_found("Unsupported target page type!".to_string())?;
            unreachable!();
        }
    };

    println!("Create post on {:?}", target_page);

    r.success("Continue implementation")
}